{% load i18n %}

<a class="navbar-item" href="{% url 'user:detail' %}">{% translate 'Preferences' %}</a>
{% if request.user.is_staff %}
    <hr class="navbar-divider">
    <a class="navbar-item" href="/admin/">Admin</a>
{% endif %}
<hr class="navbar-divider">
<a class="navbar-item" href="{% url 'user:logout' %}">{% translate 'Logout' %}</a>
